/* File:    dploader.c
   Company: Comtrol Corporation
   Author:  Craig Harrison
   Purpose: Reset Smart Hostess and start Turbo Debugger Remote if needed.
            Load a binary file into the Smart Hostess dual port RAM and
            signal the COM processor to begin execution.
   Release: 1.00: February 4, 1988 - Craig Harrison - Original
            1.01: June 23, 1991 - Craig Harrison
                  1. Add support for HOSTESS i control register initialization.
            1.02: Aug  23, 1991 - Craig Harrison
                  1. Add reset option
                  2. Add Turbo Debugger Remote startup option
            1.03: 12-3-91 - Craig Harrison
                  Changed all "outportb" to "outp" for compatability with
                  Microsoft C.
            2.0:  9-15-92 - Craig Harrison
                  Changed download to use new firmware utilities and new
                  firmware user area location (except on Smart).  Delete option
                  to strip first part of download file, this not needed with
                  new entry point having offset of 0 (Smart still using
                  default entry point of 80H, must strip 80H).  Delete
                  questions for addressing, these now defined in DPRAM.H.
                  Correct CR numbers, they were 1 less then should be.
                  Updated cr4[] for IRQ disable bit. Smart does not use CRs.
*/
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <stdlib.h>
#include <dos.h>
#include "dpram.h"
#include "firmuser.h"


FIRMUSER_T far *fu_p;                   /* ptr to firmuser user area */
int io;                                 /* controller base I/O address */

void download(void);
void reset_board(void);
int invoke_tdrem(void);

main()
{
   fu_p = (FIRMUSER_T far *)FIRMUSER_ADDR_SYS; /* firmuser area address */
   io = IO_SYS;                         /* I/O base address */
   reset_board();                       /* reset board if needed */
   download();                                                                          /* download & start ctrlpgm */
   return(0);
}

/*****************************************************************************
Purpose: Reset board, initialize control registers.
Entry: Nothing
Returns: Nothing
*/
void reset_board(void)
{
   char strbuff[81];
   int i;
   unsigned rec_flag;

   while(1)
   {
      printf("Reset Smart Hostess controller (Y/N)? ");
      scanf("%s",strbuff);
      if((*strbuff == 'y') || (*strbuff == 'Y'))
      {
         outp(io+3,0);
         delay(100);
         outp(io+3,0xff);
         printf("   Waiting for reset to complete\n");
         delay(500);
         for(i = 0;i < 21;i++)
         {
            delay(1000);
            if(fu_p->i_flag == 0x55aa)
                return;
            if(((rec_flag = fu_p->i_flag) != 0x55aa) && i >= 21)
            {
               printf("   Reset failed, interaction flag = %XH\n",rec_flag);
               exit(1);
            }
         }
      }
   }
}

/*****************************************************************************
Purpose: Load a binary file into dual port memory using the firmware
         utilities, signal the COM Processor to start execution,
         verify execution started.
Entry:   Nothing
Returns: Nothing
Notes:   The file to be downloaded must have an entry point of c0:0 and
         must write 55aaH to the interaction flag.
*/
void download(void)
{
   char far *ram_p;
   char fname[13];                          /* Name of file to download */
   unsigned wait;                           /* Do nothing loop counter */
   unsigned rec_flag;                       /* Received copy of interact flag*/
   unsigned long strip;                     /* Number bytes to strip off file*/
   static char buffer[256];                 /* Buffer for data to be xfered
                                               to dual port RAM */
   long int nbytes;                         /* Number of bytes downloaded */
   int retry,                               /* Loop control flag */
       i,
       sector,                              /* File sector beign transfered */
       f_h,                                 /* File handle */
       bytesread,                           /* Bytes read on current read */
       tdrem_invoked;                       /* Set of TDREM is invoked */

   /* Check interaction flag */
   if((rec_flag = fu_p->i_flag) != 0x55aa)
   {
      printf("Bad interaction flag = %XH before download\n",rec_flag);
      exit(1);
   }

   /* Get file name and open file for read */
   retry = 1;
   while(retry)
   {
      printf("Enter control program file name to download: ");
      scanf("%s12",fname);
      if((f_h = open(fname,O_BINARY)) == -1)
         printf("   Unable to open file %s, try again.\n",fname);
      else
         retry = 0;
   }

   /* Strip off first 80H bytes from file */
   if(-1L == lseek(f_h,0x80,SEEK_SET))
   {
      printf("   Unable to move pointer in %s, ABORT DOWNLOAD\n",fname);
      if(0 != close(f_h))
         printf("   Error attempting to close file %s\n",fname);
      exit(1);
   }

   tdrem_invoked = invoke_tdrem();     /* invoke Turbo Debugger if needed */

   /* Download file from disk to dual port RAM */
   fu_p->i_flag = 0xAA55;              /* Write to interaction flag */
   printf("Downloading %s ... \n",fname);
   nbytes = 0;
   ram_p = (char far *)LOAD_ADDR_SYS; /* control program entry point */
   while((!eof(f_h)) && (nbytes < 0xFF80))
   {
      bytesread = read(f_h,buffer,256); /* Read 256 bytes into buffer */
      nbytes = nbytes + bytesread;
      for(i = 0;i < bytesread;i++)
      {
         *ram_p++ = buffer[i]; /* Move buffer into DPRAM */
      }
   }
   if(close(f_h) != 0)
   {
      printf("   Error closing file %s after reading into DPRAM\n",fname);
      exit(1);
   }
   if(bytesread <= -1)
   {  /* Error reading file */
      printf("   Error reading file %s\n",fname);
      exit(1);
   }
        else if(nbytes >= 0xFF80)
   {  /* File to big */
      printf("   File too large to fit in one segment\n");
      exit(1);
   }
   else
   {  /* Download successful, interrupt COM Processor using firmware utility
         if TDREM not invoked */
      printf("%ld bytes downloaded successfully\n",nbytes);
      if(tdrem_invoked)
         return;
      fu_p->i_flag = 0xaa55;           /* set up interaction flag */
      outp(io + 2,0);                  /* Interrupt COM Processor */
      printf("COM processor interrupted to start control program\n");
      for(wait = 0;wait < 10;wait++)    /* wait for control program to */
      {                                 /*   restore interaction flag */
         delay(400);
         if(fu_p->i_flag == 0x55aa)
            break;
      }
      if((wait >= 10) && ((rec_flag = fu_p->i_flag) != 0x55aa))
      {
         printf("Control program failed to start, interact flag = %XH\n",
                 rec_flag);
         exit(1);
      }
      else
      {
         printf("Control program started execution\n");
      }
   }
}

/*****************************************************************************
Purpose: Invoke Turbo Debugger Remote kernel, if needed.
Entry: Nothing
Returns: 1 if tdrem is invoked, else 0
*/
int invoke_tdrem()
{
   char strbuff[0x81];                 /* Stores string entered by user */
   int do_tdrem,valid_answer,wait;
   unsigned rec_flag;                  /* Received copy of interact flag */
   unsigned far *inst_p;               /* Ptr to control pgm entry point */

   valid_answer = 0;
   while(!valid_answer)
   {
      printf("Invoke Turbo Debugger Remote Kernel on Smart Hostess controller (Y/N)? ");
      scanf("%s",strbuff);
      switch(*strbuff)
      {
         case 'y':
         case 'Y':
            valid_answer = 1;
            do_tdrem = 1;
            break;
         case 'n':
         case 'N':
            valid_answer = 1;
            do_tdrem = 0;
            break;
         default:
            break;
      }
   }
   if(do_tdrem)
   {
      inst_p = (unsigned far *)LOAD_ADDR_SYS; /* Ptr to ctrl pgm entry point */
      *inst_p = 0x27cd;                /* Write int 27H instruction used to
                                          invoke TDREM kernel */
      outp(io + 2,0);                  /* Interrupt COM Processor */
      delay(10);                       /* give it a chance to execute */
      printf("Turbo Debugger Remote Kernel Started\n");
   }
   return(do_tdrem);
}
